@@ -1,44 +1,42 @@
 """
 This module will be used to construct the surfaces and interfaces used in this package.
 """
-from OgreInterface.surfaces import Surface, Interface
-from OgreInterface import utils
-from OgreInterface.lattice_match import ZurMcGill, OgreMatch
+from copy import deepcopy
+from typing import Union, List, TypeVar, Tuple, Dict, Optional
+from itertools import combinations, product, groupby
+from collections.abc import Sequence
+import math
 
-from pymatgen.core.structure import Structure, Molecule
-from pymatgen.core.periodic_table import Element, DummySpecies
+
+from pymatgen.core.structure import Structure
 from pymatgen.core.lattice import Lattice
-from pymatgen.io.vasp.inputs import Poscar
 from pymatgen.io.ase import AseAtomsAdaptor
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 from pymatgen.core.operations import SymmOp
 from pymatgen.analysis.graphs import StructureGraph
-from pymatgen.analysis.local_env import JmolNN
-import networkx as nx
-import numpy as np
-
-
-from tqdm import tqdm
-import numpy as np
-import math
-from copy import deepcopy
-from typing import Union, List, TypeVar
-from itertools import combinations, product, groupby
-from ase import Atoms
-from ase.data import chemical_symbols
-from multiprocessing import Pool, cpu_count
-import time
-from collections.abc import Sequence
-
+from pymatgen.analysis.local_env import JmolNN, CrystalNN
 from scipy.cluster.hierarchy import fcluster, linkage
 from scipy.spatial.distance import squareform
+from ase import Atoms
+from tqdm import tqdm
+import networkx as nx
+import numpy as np
+import spglib
+
+
+from OgreInterface.surfaces import (
+    Surface,
+    Interface,
+)
+from OgreInterface import utils
+from OgreInterface.lattice_match import ZurMcGill
 
 SelfSurfaceGenerator = TypeVar(
     "SelfSurfaceGenerator", bound="SurfaceGenerator"
 )
 
-SelfOrganicSurfaceGenerator = TypeVar(
-    "SelfOrganicSurfaceGenerator", bound="OrganicSurfaceGenerator"
+SelfMolecularSurfaceGenerator = TypeVar(
+    "SelfMolecularSurfaceGenerator", bound="MolecularSurfaceGenerator"
 )
 
 SelfInterfaceGenerator = TypeVar(
@@ -75,12 +73,16 @@ class SurfaceGenerator(Sequence):
         bulk: Bulk crystal structure used to create the surface
         miller_index: Miller index of the surface
         layers: Number of layers to include in the surface
+        minimum_thickness: Optional flag to set the minimum thickness of the slab. If this is not None, then it will override the layers value
         vacuum: Size of the vacuum to include over the surface in Angstroms
+        refine_structure: Determines if the structure is first refined to it's standard settings according to it's spacegroup.
+            This is done using spglib.standardize_cell(cell, to_primitive=False, no_idealize=False). Mainly this is usefull if
+            users want to input a primitive cell of a structure instead of generating a conventional cell because most DFT people
+            work exclusively with the primitive sturcture so we always have it on hand.
         generate_all: Determines if all possible surface terminations are generated.
-        filter_ionic_slab: Determines if the terminations of ionic crystals should be filtered out based on their
-            predicted stability calculated using the IonicScoreFunction
         lazy: Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
             (this is used for the MillerIndex search to make things faster)
+        suppress_warnings: This gives the user the option to suppress warnings if they know what they are doing and don't need to see the warning messages
 
     Attributes:
         slabs (list): List of OgreInterface Surface objects with different surface terminations.
@@ -92,8 +94,6 @@ class SurfaceGenerator(Sequence):
         layers (int): Number of layers to include in the surface
         vacuum (float): Size of the vacuum to include over the surface in Angstroms
         generate_all (bool): Determines if all possible surface terminations are generated.
-        filter_ionic_slab (bool): Determines if the terminations of ionic crystals should be filtered out based on their
-            predicted stability calculated using the IonicScoreFunction
         lazy (bool): Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
             (this is used for the MillerIndex search to make things faster)
         oriented_bulk_structure (Structure): Pymatgen Structure class of the smallest building block of the slab,
@@ -112,15 +112,17 @@ class SurfaceGenerator(Sequence):
         self,
         bulk: Union[Structure, Atoms],
         miller_index: List[int],
-        layers: int,
-        vacuum: float,
-        convert_to_conventional: bool = True,
+        layers: Optional[int] = None,
+        minimum_thickness: Optional[float] = 18.0,
+        vacuum: float = 40.0,
+        refine_structure: bool = True,
         generate_all: bool = True,
-        filter_ionic_slabs: bool = False,
         lazy: bool = False,
+        suppress_warnings: bool = False,
     ) -> None:
         super().__init__()
-        self.convert_to_conventional = convert_to_conventional
+        self.refine_structure = refine_structure
+        self._suppress_warnings = suppress_warnings
 
         (
             self.bulk_structure,
@@ -136,10 +138,8 @@ class SurfaceGenerator(Sequence):
         self._point_group_operations = self._get_point_group_operations()
 
         self.miller_index = miller_index
-        self.layers = layers
         self.vacuum = vacuum
         self.generate_all = generate_all
-        self.filter_ionic_slabs = filter_ionic_slabs
         self.lazy = lazy
         (
             self.oriented_bulk_structure,
@@ -151,6 +151,13 @@ class SurfaceGenerator(Sequence):
             self.c_projection,
         ) = self._get_oriented_bulk_structure()
 
+        if layers is None and minimum_thickness is None:
+            raise "Either layer or minimum_thickness must be set"
+        if layers is not None:
+            self.layers = layers
+        if layers is None and minimum_thickness is not None:
+            self.layers = int((minimum_thickness // self.c_projection) + 1)
+
         if not self.lazy:
             self._slabs = self._generate_slabs()
         else:
@@ -181,12 +188,13 @@ class SurfaceGenerator(Sequence):
         cls,
         filename: str,
         miller_index: List[int],
-        layers: int,
-        vacuum: float,
-        convert_to_conventional: bool = True,
+        layers: Optional[int] = None,
+        minimum_thickness: Optional[float] = 18.0,
+        vacuum: float = 40.0,
+        refine_structure: bool = True,
         generate_all: bool = True,
-        filter_ionic_slabs: bool = False,
         lazy: bool = False,
+        suppress_warnings: bool = False,
     ) -> SelfSurfaceGenerator:
         """Creating a SurfaceGenerator from a file (i.e. POSCAR, cif, etc)
 
@@ -196,10 +204,13 @@ class SurfaceGenerator(Sequence):
             layers: Number of layers to include in the surface
             vacuum: Size of the vacuum to include over the surface in Angstroms
             generate_all: Determines if all possible surface terminations are generated
-            filter_ionic_slab: Determines if the terminations of ionic crystals should be filtered out based on their
-                predicted stability calculated using the IonicScoreFunction
+            refine_structure: Determines if the structure is first refined to it's standard settings according to it's spacegroup.
+                This is done using spglib.standardize_cell(cell, to_primitive=False, no_idealize=False). Mainly this is usefull if
+                users want to input a primitive cell of a structure instead of generating a conventional cell because most DFT people
+                work exclusively with the primitive structure so we always have it on hand.
             lazy: Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
                 (this is used for the MillerIndex search to make things faster)
+            suppress_warnings: This gives the user the option to suppress warnings if they know what they are doing and don't need to see the warning messages
 
         Returns:
             SurfaceGenerator
@@ -210,30 +221,81 @@ class SurfaceGenerator(Sequence):
             structure,
             miller_index,
             layers,
+            minimum_thickness,
             vacuum,
-            convert_to_conventional,
+            refine_structure,
             generate_all,
-            filter_ionic_slabs,
             lazy,
+            suppress_warnings,
         )
 
     def _get_bulk(self, atoms_or_struc):
-        if type(atoms_or_struc) == Atoms:
+        if type(atoms_or_struc) is Atoms:
             init_structure = AseAtomsAdaptor.get_structure(atoms_or_struc)
-        elif type(atoms_or_struc) == Structure:
+        elif type(atoms_or_struc) is Structure:
             init_structure = atoms_or_struc
         else:
             raise TypeError(
                 f"structure accepts 'pymatgen.core.structure.Structure' or 'ase.Atoms' not '{type(atoms_or_struc).__name__}'"
             )
 
-        if self.convert_to_conventional:
-            sg = SpacegroupAnalyzer(init_structure)
-            conventional_structure = sg.get_conventional_standard_structure()
+        if self.refine_structure:
+            conventional_structure = utils.spglib_standardize(
+                init_structure,
+                to_primitive=False,
+                no_idealize=False,
+            )
+
+            init_angles = init_structure.lattice.angles
+            init_lengths = init_structure.lattice.lengths
+            init_length_and_angles = np.concatenate(
+                [list(init_lengths), list(init_angles)]
+            )
+
+            conv_angles = conventional_structure.lattice.angles
+            conv_lengths = conventional_structure.lattice.lengths
+            conv_length_and_angles = np.concatenate(
+                [list(conv_lengths), list(conv_angles)]
+            )
+
+            if not np.isclose(
+                conv_length_and_angles - init_length_and_angles, 0
+            ).all():
+                if not self._suppress_warnings:
+                    labels = ["a", "b", "c", "alpha", "beta", "gamma"]
+                    init_cell_str = ", ".join(
+                        [
+                            f"{label} = {val:.3f}"
+                            for label, val in zip(
+                                labels, init_length_and_angles
+                            )
+                        ]
+                    )
+                    conv_cell_str = ", ".join(
+                        [
+                            f"{label} = {val:.3f}"
+                            for label, val in zip(
+                                labels, conv_length_and_angles
+                            )
+                        ]
+                    )
+                    warning_str = "\n".join(
+                        [
+                            "----------------------------------------------------------",
+                            "WARNING: The refined cell is different from the input cell",
+                            f"Initial: {init_cell_str}",
+                            f"Refined: {conv_cell_str}",
+                            "Make sure the input miller index is for the refined structure, otherwise set refine_structure=False",
+                            "To turn off this warning set suppress_warnings=True",
+                            "----------------------------------------------------------",
+                            "",
+                        ]
+                    )
+                    print(warning_str)
+
             prim_structure = self._add_symmetry_info(
                 conventional_structure, return_primitive=True
             )
-            # prim_structure = sg.get_primitive_standard_structure()
             prim_atoms = AseAtomsAdaptor.get_atoms(prim_structure)
             conventional_atoms = AseAtomsAdaptor.get_atoms(
                 conventional_structure
@@ -248,12 +310,14 @@ class SurfaceGenerator(Sequence):
         else:
             if "molecule_index" in init_structure.site_properties:
                 self._add_symmetry_info_molecule(init_structure)
+                # TODO: Primitive structure with spglib for molecular crystals
+                prim_structure = init_structure.get_primitive_structure()
             else:
-                self._add_symmetry_info(init_structure, return_primitive=False)
+                prim_structure = self._add_symmetry_info(
+                    init_structure, return_primitive=True
+                )
 
             init_atoms = AseAtomsAdaptor().get_atoms(init_structure)
-
-            prim_structure = init_structure.get_primitive_structure()
             prim_atoms = AseAtomsAdaptor().get_atoms(prim_structure)
 
             # utils._get_colored_molecules(
@@ -273,6 +337,45 @@ class SurfaceGenerator(Sequence):
         return unique_operations
 
     def _add_symmetry_info(self, struc, return_primitive=False):
+        init_lattice = struc.lattice.matrix
+        init_positions = struc.frac_coords
+        init_numbers = np.array(struc.atomic_numbers)
+        init_cell = (init_lattice, init_positions, init_numbers)
+
+        init_dataset = spglib.get_symmetry_dataset(init_cell)
+
+        struc.add_site_property(
+            "bulk_wyckoff",
+            init_dataset["wyckoffs"],
+        )
+
+        struc.add_site_property(
+            "bulk_equivalent",
+            init_dataset["equivalent_atoms"].tolist(),
+        )
+
+        if return_primitive:
+            prim_mapping = init_dataset["mapping_to_primitive"]
+            _, prim_inds = np.unique(prim_mapping, return_index=True)
+
+            prim_bulk = utils.spglib_standardize(
+                structure=struc,
+                to_primitive=True,
+                no_idealize=True,
+            )
+
+            prim_bulk.add_site_property(
+                "bulk_wyckoff",
+                [init_dataset["wyckoffs"][i] for i in prim_inds],
+            )
+            prim_bulk.add_site_property(
+                "bulk_equivalent",
+                init_dataset["equivalent_atoms"][prim_inds].tolist(),
+            )
+
+            return prim_bulk
+
+    def _add_symmetry_info_old(self, struc, return_primitive=False):
         sg = SpacegroupAnalyzer(struc)
         dataset = sg.get_symmetry_dataset()
         struc.add_site_property("bulk_wyckoff", dataset["wyckoffs"])
@@ -298,6 +401,19 @@ class SurfaceGenerator(Sequence):
             return prim_bulk
 
     def _add_symmetry_info_molecule(self, struc):
+        lattice = struc.lattice.matrix
+        positions = struc.frac_coords
+        numbers = np.array(struc.atomic_numbers)
+        cell = (lattice, positions, numbers)
+        dataset = spglib.get_symmetry_dataset(cell)
+        wyckoffs = dataset["wyckoffs"]
+        equivalent_atoms = dataset["equivalent_atoms"]
+        molecule_index = struc.site_properties["molecule_index"]
+        equivalent_molecules = [molecule_index[i] for i in equivalent_atoms]
+        struc.add_site_property("bulk_wyckoff", wyckoffs)
+        struc.add_site_property("bulk_equivalent", equivalent_molecules)
+
+    def _add_symmetry_info_molecule_old(self, struc):
         sg = SpacegroupAnalyzer(struc)
         dataset = sg.get_symmetry_dataset()
         wyckoffs = dataset["wyckoffs"]
@@ -429,10 +545,12 @@ class SurfaceGenerator(Sequence):
                     break
             # We want the indices with the maximum absolute cosine,
             # but smallest possible length.
+
             uvw, cosine, l, diff = max(
                 candidates,
                 key=lambda x: (-x[3], x[1], -x[2]),
             )
+
             basis = np.vstack([basis, uvw])
 
         init_oriented_struc = struc_for_slab.copy()
@@ -471,6 +589,7 @@ class SurfaceGenerator(Sequence):
         #         planar_mol.apply_operation(to_planar_operation)
         #         centered_mol = planar_mol.get_centered_molecule()
         #         site.properties["molecules"] = centered_mol
+        # Poscar(init_oriented_struc).write_file("POSCAR_obs_001")
 
         planar_oriented_struc = init_oriented_struc.copy()
         planar_oriented_struc.apply_operation(to_planar_operation)
@@ -605,7 +724,113 @@ class SurfaceGenerator(Sequence):
 
         return shifts
 
-    def _get_slab(self, shift=0, tol: float = 0.1, energy=None):
+    def _get_neighborhood(self, cutoff: float = 5.0) -> None:
+        obs = self.oriented_bulk_structure.copy()
+        obs.add_oxidation_state_by_guess()
+        charges = [s._oxi_state for s in obs.species]
+        self.oriented_bulk_structure.add_site_property("charge", charges)
+
+        cnn = CrystalNN(search_cutoff=cutoff, cation_anion=True)
+
+        bond_dict = {}
+        for i, site in enumerate(obs):
+            info_dict = cnn.get_nn_info(obs, i)
+            bonds = []
+            for neighbor in info_dict:
+                bond = neighbor["site"].coords - site.coords
+                bonds.append(
+                    {
+                        "from_Z": site.specie.Z,
+                        "to_Z": neighbor["site"].specie.Z,
+                        "bond": bond,
+                    }
+                )
+
+            bond_dict[site.properties["oriented_bulk_equivalent"]] = bonds
+
+        return bond_dict
+
+    def _get_surface_atoms(
+        self,
+        obs: Structure,
+        neighbor_info: Dict[int, List[Dict[str, Union[int, np.ndarray]]]],
+    ):
+        oriented_bulk_equivalents = np.array(list(neighbor_info.keys()))
+        obs_oriented_bulk_equivalent = obs.site_properties[
+            "oriented_bulk_equivalent"
+        ]
+
+        bottom_surf = []
+        top_surf = []
+        bottom_breaks = 0
+        top_breaks = 0
+        for obe in oriented_bulk_equivalents:
+            if len(neighbor_info[obe]) > 0:
+                ind = np.where(obs_oriented_bulk_equivalent == obe)[0][0]
+                center_coord = obs[ind].coords
+                bonds = np.vstack(
+                    [bond["bond"] for bond in neighbor_info[obe]]
+                )
+                bonds += center_coord
+                frac_bonds = bonds.dot(obs.lattice.inv_matrix)
+                images = np.round(frac_bonds - np.mod(frac_bonds, 1)).astype(
+                    int
+                )
+
+                bottom_broken_bonds = (images[:, -1] < 0).sum()
+                top_broken_bonds = (images[:, -1] > 0).sum()
+
+                if bottom_broken_bonds > 0:
+                    bottom_breaks += bottom_broken_bonds
+                    bottom_surf.append(obe)
+
+                if top_broken_bonds > 0:
+                    top_breaks += top_broken_bonds
+                    top_surf.append(obe)
+
+        is_top = np.isin(obs_oriented_bulk_equivalent, top_surf)
+        is_bot = np.isin(obs_oriented_bulk_equivalent, bottom_surf)
+        obs.add_site_property("is_top", is_top.tolist())
+        obs.add_site_property("is_bottom", is_bot.tolist())
+
+    def _get_surface_atoms_old(
+        self, struc: Structure, cutoff: float = 4.0
+    ) -> None:
+        obs = struc.copy()
+        obs.add_oxidation_state_by_guess()
+
+        cnn = CrystalNN(search_cutoff=cutoff)
+        top_neighborhood = []
+        bottom_neighborhood = []
+        for i in range(len(obs)):
+            info_dict = cnn.get_nn_info(obs, i)
+            top_broken_bonds = 0
+            bottom_broken_bonds = 0
+            for neighbor in info_dict:
+                if neighbor["image"][-1] > 0:
+                    top_broken_bonds += 1
+                if neighbor["image"][-1] < 0:
+                    bottom_broken_bonds += 1
+
+            if top_broken_bonds > 0:
+                top_neighborhood.append(i)
+
+            if bottom_broken_bonds > 0:
+                bottom_neighborhood.append(i)
+
+        # is_surf = np.zeros(len(obs)).astype(bool)
+        is_top = np.zeros(len(obs)).astype(bool)
+        is_bottom = np.zeros(len(obs)).astype(bool)
+
+        # is_surf[top_neighborhood + bottom_neighborhood] = True
+        is_top[top_neighborhood] = True
+        is_bottom[bottom_neighborhood] = True
+
+        # struc.add_site_property("is_surf", is_surf.tolist())
+        struc.add_site_property("is_top", is_top.tolist())
+        struc.add_site_property("is_bottom", is_bottom.tolist())
+
+    def _get_slab(self, neighbor_info, shift=0, tol: float = 0.1, energy=None):
         """
         This method takes in shift value for the c lattice direction and
         generates a slab based on the given shift. You should rarely use this
@@ -629,10 +854,43 @@ class SurfaceGenerator(Sequence):
             frac_coords=True,
             to_unit_cell=True,
         )
+        self._get_surface_atoms(slab_base, neighbor_info)
+
+        is_top = np.array(slab_base.site_properties["is_top"])
+        is_bottom = np.array(slab_base.site_properties["is_bottom"])
+        bulk_equiv = np.array(slab_base.site_properties["bulk_equivalent"])
+
+        bottom_bulk_equiv = bulk_equiv[is_bottom]
+        top_bulk_equiv = bulk_equiv[is_top]
+
+        surf_key = tuple(
+            np.concatenate(
+                [
+                    np.sort(bottom_bulk_equiv),
+                    np.sort(top_bulk_equiv),
+                    np.zeros(bottom_bulk_equiv.shape),
+                    np.ones(top_bulk_equiv.shape),
+                ]
+            ).astype(int)
+        )
 
         z_coords = slab_base.frac_coords[:, -1]
         bot_z = z_coords.min()
         top_z = z_coords.max()
+        unique_z = np.unique(np.round(z_coords, 5))
+        z_groups = [
+            (i, np.where(np.isclose(z_coords, z))[0])
+            for i, z in enumerate(unique_z)
+        ]
+        atomic_layers = np.zeros(len(z_coords))
+
+        for layer_num, inds in z_groups:
+            atomic_layers[inds] = layer_num
+
+        slab_base.add_site_property(
+            "atomic_layer_index",
+            atomic_layers.tolist(),
+        )
 
         max_z_inds = np.where(np.isclose(top_z, z_coords))[0]
 
@@ -654,12 +912,12 @@ class SurfaceGenerator(Sequence):
         )
 
         slab_base = Structure(
-            lattice=self.oriented_bulk_structure.lattice,
-            species=self.oriented_bulk_structure.species,
+            lattice=slab_base.lattice,
+            species=slab_base.species,
             coords=np.round(slab_base.frac_coords, 6),
             to_unit_cell=True,
             coords_are_cartesian=False,
-            site_properties=self.oriented_bulk_structure.site_properties,
+            site_properties=slab_base.site_properties,
         )
 
         bottom_layer_dist = np.abs(bot_z - (top_z - 1)) * init_matrix[-1, -1]
@@ -732,9 +990,10 @@ class SurfaceGenerator(Sequence):
             bottom_layer_dist,
             top_layer_dist,
             vacuum,
+            surf_key,
         )
 
-    def _generate_slabs(self):
+    def _generate_slabs(self) -> List[Surface]:
         """
         This function is used to generate slab structures with all unique
         surface terminations.
@@ -749,6 +1008,9 @@ class SurfaceGenerator(Sequence):
         non_orthogonal_slabs = []
         bottom_layer_dists = []
         top_layer_dists = []
+        surface_keys = []
+        neighbor_info = self._get_neighborhood()
+
         if not self.generate_all:
             (
                 shifted_slab_base,
@@ -757,7 +1019,11 @@ class SurfaceGenerator(Sequence):
                 bottom_layer_dist,
                 top_layer_dist,
                 actual_vacuum,
-            ) = self._get_slab(shift=possible_shifts[0])
+                surf_key,
+            ) = self._get_slab(
+                shift=possible_shifts[0],
+                neighbor_info=neighbor_info,
+            )
             orthogonal_slab.sort_index = 0
             non_orthogonal_slab.sort_index = 0
             shifted_slab_bases.append(shifted_slab_base)
@@ -765,6 +1031,7 @@ class SurfaceGenerator(Sequence):
             non_orthogonal_slabs.append(non_orthogonal_slab)
             bottom_layer_dists.append(bottom_layer_dist)
             top_layer_dists.append(top_layer_dist)
+            surface_keys.append((surf_key, 0))
         else:
             for i, possible_shift in enumerate(possible_shifts):
                 (
@@ -774,7 +1041,11 @@ class SurfaceGenerator(Sequence):
                     bottom_layer_dist,
                     top_layer_dist,
                     actual_vacuum,
-                ) = self._get_slab(shift=possible_shift)
+                    surf_key,
+                ) = self._get_slab(
+                    shift=possible_shift,
+                    neighbor_info=neighbor_info,
+                )
                 orthogonal_slab.sort_index = i
                 non_orthogonal_slab.sort_index = i
                 shifted_slab_bases.append(shifted_slab_base)
@@ -782,6 +1053,7 @@ class SurfaceGenerator(Sequence):
                 non_orthogonal_slabs.append(non_orthogonal_slab)
                 bottom_layer_dists.append(bottom_layer_dist)
                 top_layer_dists.append(top_layer_dist)
+                surface_keys.append((surf_key, i))
 
         surfaces = []
 
@@ -790,12 +1062,22 @@ class SurfaceGenerator(Sequence):
         else:
             base_structure = self.bulk_structure
 
+        sorted_surface_keys = sorted(surface_keys, key=lambda x: x[0])
+        groups = groupby(sorted_surface_keys, key=lambda x: x[0])
+
+        unique_inds = []
+        for group_key, group in groups:
+            _, inds = list(zip(*group))
+            unique_inds.append(min(inds))
+
+        unique_inds.sort()
+
         # Loop through slabs to ensure that they are all properly oriented and reduced
         # Return Surface objects
-        for i, slab in enumerate(orthogonal_slabs):
+        for i in unique_inds:
             # Create the Surface object
             surface = Surface(
-                orthogonal_slab=slab,
+                orthogonal_slab=orthogonal_slabs[i],
                 non_orthogonal_slab=non_orthogonal_slabs[i],
                 oriented_bulk=shifted_slab_bases[i],
                 bulk=base_structure,
@@ -815,9 +1097,6 @@ class SurfaceGenerator(Sequence):
 
         return surfaces
 
-    def __len__(self):
-        return len(self._slabs)
-
     @property
     def nslabs(self):
         """
@@ -835,10 +1114,10 @@ class SurfaceGenerator(Sequence):
         }
 
 
-class OrganicSurfaceGenerator(SurfaceGenerator):
-    """Class for generating surfaces from a given bulk structure.
+class MolecularSurfaceGenerator(SurfaceGenerator):
+    """Class for generating surfaces from a given bulk molecular crystal structure.
 
-    The SurfaceGenerator classes generates surfaces with all possible terminations and contains
+    The MolecularSurfaceGenerator class generates surfaces with all possible terminations and contains
     information pertinent to generating interfaces with the InterfaceGenerator.
 
     Examples:
@@ -873,8 +1152,6 @@ class OrganicSurfaceGenerator(SurfaceGenerator):
         layers (int): Number of layers to include in the surface
         vacuum (float): Size of the vacuum to include over the surface in Angstroms
         generate_all (bool): Determines if all possible surface terminations are generated.
-        filter_ionic_slab (bool): Determines if the terminations of ionic crystals should be filtered out based on their
-            predicted stability calculated using the IonicScoreFunction
         lazy (bool): Determines if the surfaces are actually generated, or if only the surface basis vectors are found.
             (this is used for the MillerIndex search to make things faster)
         oriented_bulk_structure (Structure): Pymatgen Structure class of the smallest building block of the slab,
@@ -899,7 +1176,11 @@ class OrganicSurfaceGenerator(SurfaceGenerator):
         lazy: bool = False,
     ) -> None:
         # Get the primitive structure to label the molecuels
-        prim_bulk = bulk.get_primitive_structure()
+        prim_bulk = utils.spglib_standardize(
+            bulk,
+            to_primitive=True,
+            no_idealize=True,
+        )
 
         # Get the primitive to conventional transformation matrix
         prim_to_conv = np.round(utils.conv_a_to_b(prim_bulk, bulk), 3)
@@ -917,32 +1198,22 @@ class OrganicSurfaceGenerator(SurfaceGenerator):
         # Revert back to the supercell for the input into the SurfaceGenerator
         labeled_bulk = prim_bulk.copy()
         labeled_bulk.make_supercell(prim_to_conv)
-        # dummy_bulk = utils.replace_molecules_with_atoms(labeled_bulk)
-
-        # Poscar(labeled_bulk).write_file("vis/POSCAR_mol_bulk")
-        # utils._get_colored_molecules(
-        #     labeled_bulk, "vis/POSCAR_mol_bulk_colored"
-        # )
-        # utils._get_colored_molecules(dummy_bulk, "vis/POSCAR_com_bulk_colored")
 
         super().__init__(
             bulk=labeled_bulk,
             miller_index=miller_index,
             layers=layers,
             vacuum=vacuum,
-            convert_to_conventional=False,
+            refine_structure=False,
             generate_all=generate_all,
-            filter_ionic_slabs=False,
             lazy=True,
         )
 
         # Extract the oriented bulk structure
         obs = self.oriented_bulk_structure
-        # utils._get_colored_molecules(obs, "vis/POSCAR_mol_obs_colored")
 
         # Replace the molecules with their cooresponding dummy atoms at their center of mass
         dummy_obs = utils.replace_molecules_with_atoms(obs)
-        # utils._get_colored_molecules(dummy_obs, "vis/POSCAR_com_obs_colored")
 
         # Add oriented_bulk_equivalent site property
         dummy_obs.add_site_property(
@@ -965,8 +1236,8 @@ class OrganicSurfaceGenerator(SurfaceGenerator):
         vacuum: float,
         generate_all: bool = True,
         lazy: bool = False,
-    ) -> SelfOrganicSurfaceGenerator:
-        """Creating a OrganicSurfaceGenerator from a file (i.e. POSCAR, cif, etc)
+    ) -> SelfMolecularSurfaceGenerator:
+        """Creating a MolecularSurfaceGenerator from a file (i.e. POSCAR, cif, etc)
 
         Args:
             filename: File path to the structure file
@@ -978,7 +1249,7 @@ class OrganicSurfaceGenerator(SurfaceGenerator):
                 (this is used for the MillerIndex search to make things faster)
 
         Returns:
-            OrganicSurfaceGenerator
+            MolecularSurfaceGenerator
         """
         structure = Structure.from_file(filename=filename)
 
@@ -1053,39 +1324,49 @@ class InterfaceGenerator:
 
     def __init__(
         self,
-        substrate: Surface,
-        film: Surface,
-        max_area_mismatch: float = 0.01,
-        max_angle_strain: float = 0.01,
-        max_linear_strain: float = 0.01,
-        max_area: float = 500.0,
-        interfacial_distance: Union[float, None] = 2.0,
+        substrate: Union[Surface, Interface],
+        film: Union[Surface, Interface],
+        max_strain: float = 0.01,
+        max_area_mismatch: Optional[float] = None,
+        max_area: Optional[float] = None,
+        interfacial_distance: Optional[float] = 2.0,
         vacuum: float = 40.0,
         center: bool = False,
+        substrate_strain_fraction: float = 0.0,
     ):
-        if type(substrate) == Surface:
+        if type(substrate) is Surface or type(substrate) is Interface:
             self.substrate = substrate
         else:
             raise TypeError(
-                f"InterfaceGenerator accepts 'ogre.core.Surface' not '{type(substrate).__name__}'"
+                f"InterfaceGenerator accepts 'ogre.core.Surface' or 'ogre.core.Interface' not '{type(substrate).__name__}'"
             )
 
-        if type(film) == Surface:
+        if type(film) is Surface or type(film) is Interface:
             self.film = film
         else:
             raise TypeError(
-                f"InterfaceGenerator accepts 'ogre.core.Surface' not '{type(film).__name__}'"
+                f"InterfaceGenerator accepts 'ogre.core.Surface' or 'ogre.core.Interface' not '{type(film).__name__}'"
             )
 
         self.center = center
+        self._substrate_strain_fraction = substrate_strain_fraction
         self.max_area_mismatch = max_area_mismatch
-        self.max_angle_strain = max_angle_strain
-        self.max_linear_strain = max_linear_strain
+        self.max_strain = max_strain
         self.max_area = max_area
         self.interfacial_distance = interfacial_distance
         self.vacuum = vacuum
         self.match_list = self._generate_interface_props()
 
+    def _get_point_group_operations(self, struc: Structure) -> np.ndarray:
+        sg = SpacegroupAnalyzer(struc)
+        point_group_operations = sg.get_point_group_operations(cartesian=False)
+        operation_array = np.round(
+            np.array([p.rotation_matrix for p in point_group_operations])
+        ).astype(np.int8)
+        unique_operations = np.unique(operation_array, axis=0)
+
+        return unique_operations
+
     def _generate_interface_props(self):
         zm = ZurMcGill(
             film_vectors=self.film.inplane_vectors,
@@ -1093,8 +1374,7 @@ class InterfaceGenerator:
             film_basis=self.film.uvw_basis,
             substrate_basis=self.substrate.uvw_basis,
             max_area=self.max_area,
-            max_linear_strain=self.max_linear_strain,
-            max_angle_strain=self.max_angle_strain,
+            max_strain=self.max_strain,
             max_area_mismatch=self.max_area_mismatch,
         )
         match_list = zm.run(return_all=True)
@@ -1179,9 +1459,9 @@ class InterfaceGenerator:
             sorted_matches = sorted(
                 unique_matches,
                 key=lambda x: (
-                    x.area,
-                    np.max(np.abs(x.linear_strain)),
-                    np.abs(x.angle_strain),
+                    round(x.area, 6),
+                    round(x.strain, 6),
+                    round(x._rotation_distortion, 6),
                 ),
             )
 
@@ -1250,6 +1530,7 @@ class InterfaceGenerator:
             match=match,
             vacuum=self.vacuum,
             center=self.center,
+            substrate_strain_fraction=self._substrate_strain_fraction,
         )
         return interface
 
@@ -1264,7 +1545,9 @@ class InterfaceGenerator:
 
         interfaces = []
 
-        print("Generating Interfaces:")
+        print(
+            f"Generating Interfaces for Film ({self.film.termination_index}) and Substrate ({self.substrate.termination_index}):"
+        )
         for match in tqdm(self.match_list, dynamic_ncols=True):
             interface = Interface(
                 substrate=self.substrate,
@@ -1273,6 +1556,7 @@ class InterfaceGenerator:
                 match=match,
                 vacuum=self.vacuum,
                 center=self.center,
+                substrate_strain_fraction=self._substrate_strain_fraction,
             )
             interfaces.append(interface)
 
