@@ -1,12 +1,9 @@
-from OgreInterface.generate import SurfaceGenerator
-from OgreInterface.lattice_match import ZurMcGill
-from OgreInterface import utils
+from itertools import product
+from typing import Union, Optional
 
 from pymatgen.core.structure import Structure
 from pymatgen.io.ase import AseAtomsAdaptor
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
-from pymatgen.core.surface import get_symmetrically_distinct_miller_indices
-
 from ase import Atoms
 import numpy as np
 
@@ -15,10 +12,9 @@ from matplotlib.collections import PatchCollection
 from matplotlib.colors import Normalize
 from mpl_toolkits.axes_grid1.axes_divider import make_axes_locatable
 
-from functools import reduce
-from itertools import product
-
-from typing import Union
+from OgreInterface.generate import SurfaceGenerator
+from OgreInterface.lattice_match import ZurMcGill
+from OgreInterface import utils
 
 
 class MillerSearch(object):
@@ -39,8 +35,10 @@ class MillerSearch(object):
         max_angle_strain: Angle strain tolerance for the InterfaceGenerator
         max_linear_strain: Lattice vectors length mismatch tolerance for the InterfaceGenerator
         max_area: Maximum area of the matched supercells
-        convert_to_conventional: Determines if the input structure is converted to it's conventional standard structure according to
-            the Pymatgen SpacegroupAnalyzer.
+        refine_structure: Determines if the structure is first refined to it's standard settings according to it's spacegroup.
+            This is done using spglib.standardize_cell(cell, to_primitive=False, no_idealize=False). Mainly this is usefull if
+            users want to input a primitive cell of a structure instead of generating a conventional cell because most DFT people
+            work exclusively with the primitive structure so we always have it on hand.
 
     Attributes:
         substrate (Structure): Pymatgen Structure of the substrate
@@ -51,8 +49,10 @@ class MillerSearch(object):
         max_angle_strain (float): Angle strain tolerance for the InterfaceGenerator
         max_linear_strain (float): Lattice vectors length mismatch tolerance for the InterfaceGenerator
         max_area (float): Maximum area of the matched supercells
-        convert_to_conventional (bool): Determines if the input structure is converted to it's conventional standard structure according to
-            the Pymatgen SpacegroupAnalyzer.
+        refine_structure: Determines if the structure is first refined to it's standard settings according to it's spacegroup.
+            This is done using spglib.standardize_cell(cell, to_primitive=False, no_idealize=False). Mainly this is usefull if
+            users want to input a primitive cell of a structure instead of generating a conventional cell because most DFT people
+            work exclusively with the primitive structure so we always have it on hand.
         substrate_inds (list): List of unique substrate surface miller indices
         film_inds (list): List of unique film surface miller indices
     """
@@ -63,19 +63,21 @@ class MillerSearch(object):
         film: Union[Structure, Atoms, str],
         max_substrate_index: int = 1,
         max_film_index: int = 1,
-        max_area_mismatch: float = 0.01,
-        max_angle_strain: float = 0.01,
-        max_linear_strain: float = 0.01,
-        max_area: float = 500.0,
-        convert_to_conventional: bool = True,
+        max_strain: float = 0.01,
+        max_area_mismatch: Optional[float] = None,
+        max_area: Optional[float] = None,
+        refine_structure: bool = True,
+        suppress_warnings: bool = False,
     ) -> None:
-        self.convert_to_conventional = convert_to_conventional
-        if type(substrate) == str:
+        self.refine_structure = refine_structure
+        self._suppress_warnings = suppress_warnings
+
+        if type(substrate) is str:
             self.substrate, _ = self._get_bulk(Structure.from_file(substrate))
         else:
             self.substrate, _ = self._get_bulk(substrate)
 
-        if type(film) == str:
+        if type(film) is str:
             self.film, _ = self._get_bulk(Structure.from_file(film))
         else:
             self.film, _ = self._get_bulk(film)
@@ -83,8 +85,7 @@ class MillerSearch(object):
         self.max_film_index = max_film_index
         self.max_substrate_index = max_substrate_index
         self.max_area_mismatch = max_area_mismatch
-        self.max_angle_strain = max_angle_strain
-        self.max_linear_strain = max_linear_strain
+        self.max_strain = max_strain
         self.max_area = max_area
         self.substrate_inds = self._get_unique_miller_indices(
             self.substrate, self.max_substrate_index
@@ -98,48 +99,81 @@ class MillerSearch(object):
         self._area_data = None
 
     def _get_bulk(self, atoms_or_struc):
-        if type(atoms_or_struc) == Atoms:
+        if type(atoms_or_struc) is Atoms:
             init_structure = AseAtomsAdaptor.get_structure(atoms_or_struc)
-        elif type(atoms_or_struc) == Structure:
+        elif type(atoms_or_struc) is Structure:
             init_structure = atoms_or_struc
         else:
             raise TypeError(
                 f"structure accepts 'pymatgen.core.structure.Structure' or 'ase.Atoms' not '{type(atoms_or_struc).__name__}'"
             )
 
-        if self.convert_to_conventional:
-            sg = SpacegroupAnalyzer(init_structure)
-            conventional_structure = sg.get_conventional_standard_structure()
-            prim_structure = sg.get_primitive_standard_structure()
+        if self.refine_structure:
+            conventional_structure = utils.spglib_standardize(
+                init_structure,
+                to_primitive=False,
+                no_idealize=False,
+            )
 
-            return (conventional_structure, prim_structure)
+            init_angles = init_structure.lattice.angles
+            init_lengths = init_structure.lattice.lengths
+            init_length_and_angles = np.concatenate(
+                [list(init_lengths), list(init_angles)]
+            )
+
+            conv_angles = conventional_structure.lattice.angles
+            conv_lengths = conventional_structure.lattice.lengths
+            conv_length_and_angles = np.concatenate(
+                [list(conv_lengths), list(conv_angles)]
+            )
+
+            if not np.isclose(
+                conv_length_and_angles - init_length_and_angles, 0
+            ).all():
+                if not self._suppress_warnings:
+                    labels = ["a", "b", "c", "alpha", "beta", "gamma"]
+                    init_cell_str = ", ".join(
+                        [
+                            f"{label} = {val:.3f}"
+                            for label, val in zip(
+                                labels, init_length_and_angles
+                            )
+                        ]
+                    )
+                    conv_cell_str = ", ".join(
+                        [
+                            f"{label} = {val:.3f}"
+                            for label, val in zip(
+                                labels, conv_length_and_angles
+                            )
+                        ]
+                    )
+                    warning_str = "\n".join(
+                        [
+                            "----------------------------------------------------------",
+                            "WARNING: The refined cell is different from the input cell",
+                            f"Initial: {init_cell_str}",
+                            f"Refined: {conv_cell_str}",
+                            "Make sure the input miller index is for the refined structure, otherwise set refine_structure=False",
+                            "To turn off this warning set suppress_warnings=True",
+                            "----------------------------------------------------------",
+                            "",
+                        ]
+                    )
+                    print(warning_str)
+
+            conventional_atoms = AseAtomsAdaptor.get_atoms(
+                conventional_structure
+            )
+
+            return (
+                conventional_structure,
+                conventional_atoms,
+            )
         else:
-            prim_structure = init_structure.get_primitive_structure()
+            init_atoms = AseAtomsAdaptor().get_atoms(init_structure)
 
-            return init_structure, prim_structure
-
-    # def _get_bulk(self, atoms_or_struc):
-    #     if type(atoms_or_struc) == Atoms:
-    #         init_structure = AseAtomsAdaptor.get_structure(atoms_or_struc)
-    #     elif type(atoms_or_struc) == Structure:
-    #         init_structure = atoms_or_struc
-    #     else:
-    #         raise TypeError(
-    #             f"structure accepts 'pymatgen.core.structure.Structure' or 'ase.Atoms' not '{type(atoms_or_struc).__name__}'"
-    #         )
-
-    #     if self.convert_to_conventional:
-    #         sg = SpacegroupAnalyzer(init_structure)
-    #         conventional_structure = sg.get_conventional_standard_structure()
-    #         conventional_atoms = AseAtomsAdaptor.get_atoms(
-    #             conventional_structure
-    #         )
-
-    #         return conventional_structure, conventional_atoms
-    #     else:
-    #         init_atoms = AseAtomsAdaptor().get_atoms(init_structure)
-
-    #         return init_structure, init_atoms
+            return init_structure, init_atoms
 
     def _float_gcd(self, a, b, rtol=1e-05, atol=1e-08):
         t = min(abs(a), abs(b))
@@ -253,7 +287,7 @@ class MillerSearch(object):
                 vacuum=10,
                 generate_all=False,
                 lazy=True,
-                convert_to_conventional=self.convert_to_conventional,
+                refine_structure=self.refine_structure,
             )
             sub_inplane_vectors = sg_sub.inplane_vectors
             sub_area = np.linalg.norm(
@@ -271,7 +305,7 @@ class MillerSearch(object):
                 vacuum=10,
                 generate_all=False,
                 lazy=True,
-                convert_to_conventional=self.convert_to_conventional,
+                refine_structure=self.refine_structure,
             )
             film_inplane_vectors = sg_film.inplane_vectors
             film_area = np.linalg.norm(
@@ -291,8 +325,7 @@ class MillerSearch(object):
                     film_basis=film[2],
                     substrate_basis=substrate[2],
                     max_area=self.max_area,
-                    max_linear_strain=self.max_linear_strain,
-                    max_angle_strain=self.max_angle_strain,
+                    max_strain=self.max_strain,
                     max_area_mismatch=self.max_area_mismatch,
                 )
                 matches = zm.run()
@@ -300,8 +333,8 @@ class MillerSearch(object):
                 if len(matches) > 0:
                     min_area_match = matches[0]
                     area = min_area_match.area
-                    strain = min_area_match.linear_strain
-                    misfits[i, j] = np.abs(strain).max()
+                    strain = min_area_match.strain
+                    misfits[i, j] = strain
                     areas[i, j] = area / np.sqrt(substrate[1] * film[1])
 
         self.misfits = np.round(misfits.T, 8)
@@ -309,7 +342,7 @@ class MillerSearch(object):
 
     def plot_misfits(
         self,
-        cmap: str = "magma",
+        cmap: str = "coolwarm",
         dpi: int = 400,
         output: str = "misfit_plot.png",
         fontsize: float = 12.0,
@@ -374,16 +407,27 @@ class MillerSearch(object):
         )
 
         if film_label is None:
-            film_label = self.film.composition.reduced_formula
+            film_label = utils.get_latex_formula(
+                self.film.composition.reduced_formula
+            )
 
         ax.set_ylabel(film_label + " Miller Index", fontsize=fontsize)
 
         if substrate_label is None:
-            substrate_label = self.substrate.composition.reduced_formula
+            substrate_label = utils.get_latex_formula(
+                self.substrate.composition.reduced_formula
+            )
 
         ax.set_xlabel(substrate_label + " Miller Index", fontsize=fontsize)
 
-        R = 0.85 * s / np.nanmax(s) / 2
+        if not np.isnan(s).all():
+            R = 0.85 * s / np.nanmax(s) / 2
+        else:
+            print(
+                "WARNING: No matches were found with the current settings. Try increasing max_area or max_strain"
+            )
+            R = s
+
         circles = [
             plt.Circle((i, j), radius=r, edgecolor="black", lw=3)
             for r, i, j in zip(R.flat, x.flat, y.flat)
@@ -393,8 +437,8 @@ class MillerSearch(object):
             array=c.flatten(),
             cmap=cmap,
             norm=Normalize(
-                vmin=np.max([np.nanmin(c) - 0.01 * np.nanmin(c), 0]),
-                vmax=np.nanmax(c) + 0.01 * np.nanmax(c),
+                vmin=0.0,
+                vmax=100 * self.max_strain,
             ),
             edgecolor="black",
             linewidth=1,
@@ -414,7 +458,7 @@ class MillerSearch(object):
         ax.grid(which="minor", linestyle=":", linewidth=0.75)
 
         cbar = fig.colorbar(col, cax=cax)
-        cbar.set_label("Misfit Percentage", fontsize=fontsize)
+        cbar.set_label("Strain (%)", fontsize=fontsize)
         cbar.ax.tick_params(labelsize=fontsize)
         cbar.ax.ticklabel_format(
             style="sci", scilimits=(-3, 3), useMathText=True
@@ -423,7 +467,7 @@ class MillerSearch(object):
 
         ax.set_aspect("equal")
         fig.tight_layout(pad=0.4)
-        fig.savefig(output, bbox_inches="tight")
+        fig.savefig(output, bbox_inches="tight", transparent=False)
 
         if not show_in_colab:
             plt.close(fig)
