@@ -1,19 +1,21 @@
 from __future__ import annotations
-import numpy as np
 import copy
 from functools import reduce
+import itertools
+import functools
+import math
+import collections
+
 from pymatgen.core.structure import Structure, Molecule
 from pymatgen.core.lattice import Lattice
 from pymatgen.io.ase import AseAtomsAdaptor
 from pymatgen.io.vasp.inputs import Poscar
 from pymatgen.core.periodic_table import DummySpecies
-import itertools
-import functools
-import math
-import collections
 from pymatgen.analysis.graphs import StructureGraph
 from pymatgen.analysis.local_env import JmolNN
+import numpy as np
 import networkx as nx
+import spglib
 
 
 def _get_colored_molecules(struc, output):
@@ -30,6 +32,87 @@ def _get_colored_molecules(struc, output):
     Poscar(colored_struc).write_file(output)
 
 
+def get_latex_formula(formula: str) -> str:
+    groups = itertools.groupby(formula, key=lambda x: x.isdigit())
+
+    latex_formula = ""
+    for k, group in groups:
+        if k:
+            part = "$_{" + "".join(list(group)) + "}$"
+        else:
+            part = "".join(list(group))
+
+        latex_formula += part
+
+    return latex_formula
+
+
+def apply_strain_matrix(
+    structure: Structure, strain_matrix: np.ndarray
+) -> Structure:
+    """
+    This function applies a strain matrix to a structure to match it to another lattice
+    i.e. straining a film to match with the substrate material. The strain_matrix can be calculated
+    by the following equation:
+        strain_matrix = np.linalg.inv(old_lattice_matrix) @ new_lattice_matrix
+    """
+    new_matrix = structure.lattice.matrix @ strain_matrix
+
+    strained_structure = Structure(
+        lattice=Lattice(new_matrix),
+        species=structure.species,
+        coords=structure.frac_coords,
+        to_unit_cell=True,
+        coords_are_cartesian=False,
+        site_properties=structure.site_properties,
+    )
+
+    return strained_structure
+
+
+def spglib_standardize(
+    structure: Structure,
+    to_primitive: bool = False,
+    no_idealize: bool = False,
+) -> Structure:
+    """
+    This function standardized a given structure using the spglib library
+
+    Args:
+        structure: Input pymatgen Structure
+        to_primitive: Determines if the structure should be converted to it's primitive unit cell
+        no_idealize: Determines if the lattice vectors should be idealized
+            (i.e. rotate a cubic structure so \\vec{a} points along the [1, 0, 0] cartesian direction)
+
+    Returns:
+        The standardized structure in the form of a pymatgen Structure object
+    """
+    init_lattice = structure.lattice.matrix
+    init_positions = structure.frac_coords
+    init_numbers = np.array(structure.atomic_numbers)
+    init_cell = (init_lattice, init_positions, init_numbers)
+
+    (
+        standardized_lattice,
+        standardized_positions,
+        standardized_numbers,
+    ) = spglib.standardize_cell(
+        init_cell,
+        to_primitive=to_primitive,
+        no_idealize=no_idealize,
+    )
+
+    standardized_structure = Structure(
+        lattice=Lattice(standardized_lattice),
+        species=standardized_numbers,
+        coords=standardized_positions,
+        to_unit_cell=True,
+        coords_are_cartesian=False,
+    )
+
+    return standardized_structure
+
+
 def apply_op_to_mols(struc, op):
     for site in struc:
         mol = site.properties["molecules"]
@@ -204,6 +287,12 @@ def get_layer_supercelll(
     }
     new_site_properties["layer_index"] = sc_layer_inds.tolist()
 
+    if "atomic_layer_index" in new_site_properties:
+        atomic_layers = np.array(new_site_properties["atomic_layer_index"])
+        offset = (atomic_layers.max() * sc_layer_inds) + sc_layer_inds
+        new_atomic_layers = atomic_layers + offset
+        new_site_properties["atomic_layer_index"] = new_atomic_layers
+
     layer_transform = np.eye(3)
     layer_transform[-1, -1] = layers + vacuum_scale
     layer_matrix = layer_transform @ structure.lattice.matrix
@@ -224,7 +313,7 @@ def group_layers(structure, atol=None):
     """
     This function will find the atom indices belonging to each unique atomic layer.
 
-    Parameters:
+    Args:
         structure (pymatgen.core.structure.Structure): Slab structure
         atol (float or None): Tolarence used for grouping the layers. Useful for grouping
             layers in a structure with relaxed atomic positions.
@@ -274,7 +363,7 @@ def get_reduced_basis(basis: np.ndarray) -> np.ndarray:
     |  0 -2  4 | ==> | 0 -1  2 |
     | 10 10 10 |     | 1  1  1 |
 
-    Parameters:
+    Args:
         basis (np.ndarray): 3x3 matrix defining the lattice vectors
 
     Returns:
